/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package Resistorcalculation;

//import java.text.DecimalFormat;
//import java.text.DecimalFormatSymbols;

/**
 *
 * @author Sun2k
 */
public class ResistorCalculation {

    private static float[] band4Multiplier = { 0, 0.01f, 0.1f, 1, 10, 100, 1000,
            10000, 100000, 1000000, 10000000, 100000000, 1000000000};

    private static float[] band4Tolerance = {20, 10, 5, 0, 1, 2, 0, 0, 0.5f, 0.25f,
                                0.1f, 0, 0};

    private static float[] band5Multiplier = { 0, 0.01f, 0.1f, 1, 10, 100, 1000,
            10000, 100000, 1000000, 0, 0, 0 };

    private static float[] band5Tolerance = {0, 0, 0, 0, 1, 2, 0, 0, 0.5f, 0.25f,
                                0.1f, 0.05f, 0};

    public static float[] band6TempCoeff = {0, 0, 0, 200e-6f, 100e-6f,
            50e-6f, 15e-6f, 25e-6f, 0, 10e-6f, 5e-6f, 0, 0};


    /*--------------------------------------------------------------------------
     * Resistor calculation with 4 bands
     *------------------------------------------------------------------------*/

    
    public static float getResistance4(int band1, int band2,
                         int band3, int band4)
    {
       
        // check given resistance values for validity
        if (band1 < ResistorColor.BROWN || band2 < ResistorColor.BLACK ||
            band3 == ResistorColor.NONE)
            return 0;
        /*
        // check given tolerance value for validity
        if (band4 == ResistorColor.BLACK || band4 == ResistorColor.ORANGE ||
            band4 == ResistorColor.YELLOW || band4 == ResistorColor.GRAY ||
            band4 == ResistorColor.WHITE)
            return 0;
        */
        
        // calc resistance value
        return (band1 * 10 + band2) * band4Multiplier[band3 + 3];
    }

    public static float getTolerance4(int band4)
    {
        // check given tolerance value for validity
        if (band4 == ResistorColor.BLACK || band4 == ResistorColor.ORANGE ||
            band4 == ResistorColor.YELLOW || band4 == ResistorColor.GRAY ||
            band4 == ResistorColor.WHITE)
            return 0;
        else
            return band4Tolerance[band4 + 3];
    }

    public static ToleranceIntervall getToleranceIntervall4(int band1,
            int band2, int band3, int band4)
    {
        ToleranceIntervall tolInt = new ToleranceIntervall();

        float res = getResistance4(band1, band2, band3, band4);
        tolInt.lowerbound = res - (getTolerance4(band4) * res / 100);
        tolInt.upperbound = res + (getTolerance4(band4) * res / 100);
        return tolInt;
    }

    public static String getToleranceString4(int band1,
            int band2, int band3, int band4)
    {
        ToleranceIntervall intervall =
                getToleranceIntervall4(band1, band2, band3, band4);
        return getTolerance4(band4) + " (" + intervall.lowerbound +
                " , " + intervall.upperbound + ")";
    }


    /*--------------------------------------------------------------------------
     * Resistor calculation with 5 or 6 bands
     *------------------------------------------------------------------------*/

    public static float getResistance5(int band1, int band2,
                         int band3, int band4, int band5)
    {

        // check given resistance values for validity
        if (band1 < ResistorColor.BROWN || band2 < ResistorColor.BLACK ||
            band3 < ResistorColor.BLACK)
            return 0;
        /*
        if (band4 < ResistorColor.SILVER || band4 > ResistorColor.BLUE)
            return 0;

        // check given tolerance value for validity
        if (band5 < ResistorColor.BLACK || band5 == ResistorColor.ORANGE ||
            band5 == ResistorColor.YELLOW || band5 == ResistorColor.WHITE)
            return 0;
        */

        // calc resistance value
        return (band1 * 100 + band2 * 10 + band3) * band5Multiplier[band4 + 3];
    }

    public static float getTolerance5(int band5)
    {
        // check given tolerance value for validity
        if (band5 < ResistorColor.BLACK || band5 == ResistorColor.ORANGE ||
            band5 == ResistorColor.YELLOW || band5 == ResistorColor.WHITE)
            return 0;
        else
            return band5Tolerance[band5 + 3];
    }

    public static float getTempCoeff(int band6)
    {
        if (band6 < ResistorColor.BLACK || band6 == ResistorColor.GREEN ||
            band6 > ResistorColor.PURPLE)
            return 0;
        else
            return band6TempCoeff[band6 + 3];
    }

    public static float getTemp(int index) {
        return band6TempCoeff[index + 3];
     }

    public static ToleranceIntervall getToleranceIntervall5(int band1,
            int band2, int band3, int band4, int band5)
    {
        ToleranceIntervall tolInt = new ToleranceIntervall();

        float res = getResistance5(band1, band2, band3, band4, band5);
        tolInt.lowerbound = res - (getTolerance5(band5) * res / 100);
        tolInt.upperbound = res + (getTolerance5(band5) * res / 100);
        return tolInt;
    }

    public static String getToleranceString5(int band1,
            int band2, int band3, int band4, int band5)
    {
        ToleranceIntervall intervall =
                getToleranceIntervall5(band1, band2, band3, band4, band5);
        return getTolerance5(band5) + " (" + intervall.lowerbound +
                " , " + intervall.upperbound + ")";
    }

    /*--------------------------------------------------------------------------
     * Get resistor from specified resistor
     *------------------------------------------------------------------------*/
    public static Resistor getResistor5(float resistance)
    {
        Resistor rstr = new Resistor();
        // resistanceSaved = resistance;
        int multiplier = 0;

        // check number of digits behind a decimal point
        if (resistance != (int)resistance)
        {
            while (resistance != (int)resistance)
            {
                multiplier--;
                resistance *= 10;
            }
        }

        // check out number of digits before a decimal point
        if (resistance > 1000)
        {
            while (resistance > 1000)
            {
                multiplier++;
                resistance /= 10;
            }
        }
        else if (resistance < 10)
        {
            while (resistance < 10)
            {
                multiplier--;
                resistance *= 10;
            }
        }

        if (multiplier < -2 || multiplier > 9)
            return null;
        rstr.bands[3] = multiplier;

        if (resistance < 100)
            rstr.bands[0] = ResistorColor.NONE;
        else
            rstr.bands[0] = (int)(resistance / 100);

        resistance %= 100;

        if (resistance < 10)
            rstr.bands[1] = (int)resistance;
        else
            rstr.bands[1] = (int)(resistance / 10);

        rstr.bands[2] = (int)(resistance % 10);

       
        return rstr;
    }



    public static Resistor getResistor4(float resistance)
    {
        Resistor rstr = new Resistor();
        int multiplier = 0;

        // check number of digits behind a decimal point
        if (resistance != (int)resistance)
        {
            while (resistance != (int)resistance)
            {
                multiplier--;
                resistance *= 10;
            }
        }

        // check out number of digits before a decimal point
        if (resistance > 100)
        {
            while (resistance > 100)
            {
                multiplier++;
                resistance /= 10;
            }
        }
        else if (resistance < 10)
        {
            while (resistance < 10)
            {
                multiplier--;
                resistance *= 10;
            }
        }

        if (multiplier < -2 || multiplier > 9)
            return null;
        rstr.bands[2] = multiplier;

        /*if (resistance < 100)
            rstr.bands[0] = ResistorColor.NONE;
        else
            rstr.bands[0] = (int)(resistance / 100);

        resistance %= 100;*/

        if (resistance < 10)
            rstr.bands[0] = (int)resistance;
        else
            rstr.bands[0] = (int)(resistance / 10);

        rstr.bands[1] = (int)(resistance % 10);

        return rstr;
    }

}
