/*
 * Main.fx
 *
 * Created on 26.08.2009, 18:41:28
 *
 * A simple applet for calculating Resistors
 * www.sunshine2k.de
 * webmaster@sunshine2k.de
 */

package resistorapplet;

import javafx.stage.Stage;
import javafx.scene.Scene;
import javafx.scene.text.Font;
import resistorapplet.ResistorShape;
import javafx.scene.control.Label;
import javafx.scene.control.TextBox;
import javafx.scene.input.MouseEvent;
import javafx.scene.control.RadioButton;
import javafx.scene.control.ToggleGroup;
import java.lang.NumberFormatException;
import Resistorcalculation.*;

/**
 * @author Sunshine2k
 */

// reference to available band colors
def mycolors = ResistorBandShape.Colors;

def ColorStrings = ["None", "SILVER", "GOLD", "BLACK", "BROWN", "RED", "ORANGE",
        "YELLOW", "GREEN", "BLUE", "PURPLE", "GRAY", "WHITE"];

// the actual resistor shape
def MyResistorShape = ResistorShape  {
    translateX : 20
        }

// reference to the six color bands of the resistor
var Bands = MyResistorShape.Bands;

// the labels with the colors as word
var ColorLabels : Label[] =
    for (i in [0..<6])
        Label {
            width: 80,
            height : 25,
            translateX: 120 + i * 55,
            translateY: 115,
            text: bind ColorStrings[Bands[i].colorIndex];
            font : Font { size : 10; }
            textFill : bind mycolors[Bands[i].colorIndex];
        }

// here we handle clicks on the radiobuttons
public function HandleRadioButtons() : Void
{
    if (Band4RadioButton.selected)
    {
        Bands[4].visible = false;
        Bands[5].visible = false;
        Bands[4].colorIndex = 0;
        Bands[5].colorIndex = 0;
    }
    if (Band5RadioButton.selected)
    {
        Bands[4].visible = true;
        Bands[5].visible = false;
        Bands[5].colorIndex = 0;
    }
    if (Band6RadioButton.selected)
    {
        Bands[4].visible = true;
        Bands[5].visible = true;
    }
}

// calculate the resistance value when user changed a color of one band
public function UpdateBands() : Void
{
    if (Band4RadioButton.selected)
    {
        //println("1 {Bands[0].colorCode} 2 {Bands[1].colorCode} 3 {Bands[2].colorCode} 4 {Bands[3].colorCode}");
        ResultField.text = ResistorCalculation.getResistance4(
                Bands[0].colorCode, Bands[1].colorCode,
                Bands[2].colorCode,Bands[3].colorCode).toString() ;
    }
    else
    {
        ResultField.text = ResistorCalculation.getResistance5(
                Bands[0].colorCode, Bands[1].colorCode, Bands[2].colorCode,
                Bands[3].colorCode, Bands[4].colorCode).toString() ;
    }
}


// the radio buttons
def BandGroup = ToggleGroup { }

var Band4RadioButton : RadioButton = RadioButton {
    toggleGroup: BandGroup
    text: "4 Bands"
    layoutX: 140
    layoutY: 150
    onMouseClicked: function(evt : MouseEvent) : Void {
        HandleRadioButtons();
    }
}
var Band5RadioButton : RadioButton = RadioButton {
    toggleGroup: BandGroup
    text: "5 Bands"
    layoutX: 220
    layoutY: 150
    onMouseClicked: function(evt : MouseEvent) : Void {
        HandleRadioButtons();
    }
}
var Band6RadioButton : RadioButton = RadioButton {
    toggleGroup: BandGroup
    text: "6 Bands"
    layoutX: 300
    layoutY: 150
    selected:true
    onMouseClicked: function(evt : MouseEvent) : Void {
        HandleRadioButtons();
    }

}

// string holding the tolerance information of current resistor bands
var ToleranceFromBands = bind {

    if (Band4RadioButton.selected)
    {
        ResistorCalculation.getToleranceString4(Bands[0].colorCode, Bands[1].colorCode,
            Bands[2].colorCode,Bands[3].colorCode);
    }
    else
    {
         ResistorCalculation.getToleranceString5(Bands[0].colorCode, Bands[1].colorCode,
            Bands[2].colorCode, Bands[3].colorCode, Bands[4].colorCode);
    }

}

// string holding the temperature coefficient value of current resistor bands
var TempCoeffFromBands = bind {

    if (Band6RadioButton.selected)
    {
        ResistorCalculation.getTempCoeff(Bands[5].colorCode).toString();
    }
    else
    {
        ""
    }
}



var ResultTextLabel : Label = Label {
    text: "Resistance:"
    translateX: 110,
    translateY: 180,
    width : 70,
    height : 20
}

var OmegaLabel : Label = Label {
    text : "Ω",
    font : Font {
        size: 14
    }
    translateX: 280,
    translateY: 180,
    width : 20,
    height : 20,
    visible : true
}




var ResultField : TextBox = TextBox {
    editable: true
    translateX: 200,
    translateY: 180,
    width : 70,
    height : 20,
    focusTraversable: false,
    text: "";
    visible : true;

    action: function() : Void
    {
       var rt : Resistor;

       try {

           if (Band4RadioButton.selected)
           {
               rt = ResistorCalculation.getResistor4(Float.parseFloat(ResultField.text));
           }
           else
           {
               rt = ResistorCalculation.getResistor5(Float.parseFloat(ResultField.text));
           }

           println(rt.toString());
           // set color
           for (i in [0..5])
           {
              Bands[i].currentColor = mycolors[rt.bands[i]+3];
           }

           // update internal variables of bands
           for (i in [0..5])
           {
              Bands[i].colorIndex = rt.bands[i]+3;
           }

           for (i in [0..5])
           {
              Bands[i].colorCode = ResistorBandShape.ColorCode[Bands[i].colorIndex];
           }

           UpdateBands();

       }
       catch (nfe : NumberFormatException) {
           ResultField.text = "Invalid number";
       }
    }
}


var ToleranceTextLabel : Label = Label {
        text: "Tolerance:",
        translateX: 110,
        translateY: 200,
        width: 80,
        height: 25
}

var ToleranceLabel : Label = Label {
        text: bind ToleranceFromBands,
        translateX: 200,
        translateY: 200
        width: 230,
        height: 25
}

var TempCoeffTextLabel : Label = Label {
        text: "Temp. Coeff.:",
        translateX: 110,
        translateY: 220,
        width: 80,
        height: 25
}

var TempCoeffLabel : Label = Label {
        text: bind TempCoeffFromBands,
        translateX: 200,
        translateY: 220,
        width: 120,
        height: 25,
        onMouseClicked: function (evt: MouseEvent) : Void {
            //println("Tempcoeff {TempCoeffLabel.text}")
        }

}

public function run(args: String[])
{
    Stage {
        title: "Application title"
        width: 550
        height: 340
        scene: Scene {
            content: [

                MyResistorShape,

                ColorLabels,
                Band4RadioButton, Band5RadioButton, Band6RadioButton,

                ResultTextLabel,
                ResultField,
                OmegaLabel,

                ToleranceTextLabel,
                ToleranceLabel,

                TempCoeffTextLabel,
                TempCoeffLabel
            ]
        }
    }
}