/*
 * ResistorShape.fx
 *
 * Created on 26.08.2009, 18:41:59
 */

package resistorapplet;

import javafx.scene.CustomNode;
import javafx.scene.Node;
import javafx.scene.Group;
import javafx.scene.shape.*;
import javafx.scene.paint.*;
import javafx.scene.effect.*;
import javafx.scene.paint.LinearGradient;
import javafx.animation.transition.PathTransition;
import javafx.animation.transition.*;
import javafx.animation.Interpolator;
import javafx.animation.Timeline;

/**
 * @author Sun2k
 */



public class ResistorShape extends CustomNode {

    public def circles : Circle[] = for (n in [1..2]) {
        Circle {
        centerX: 60;
        centerY : 70;
        radius : 10,
        fill : RadialGradient {
            centerX: 60, centerY: 70,
            radius: 10,
            proportional: false

            stops: [
                Stop {offset: 0.0 color: Color.WHITE},
                Stop {offset: 1.0 color: Color.LIGHTYELLOW},
            ]
        }
        effect: GaussianBlur { radius: 10}
        }
    }

    def animationPathLower : Path = Path {
        elements: [
            MoveTo { x : 0 y: 70  }
            LineTo { x : 60 y: 70 }
            QuadCurveTo { x: 130 y : 100 controlX: 60 controlY: 130  }
            LineTo { x : 370, y : 100 }
            QuadCurveTo { x : 440 y : 70 controlX: 440 controlY: 130}
            LineTo { x : 500 y: 70 }
        ]
    }

    def animationPathUpper : Path = Path {
        elements: [
            MoveTo { x : 0 y: 70  }
            LineTo { x : 60 y: 70 }
            QuadCurveTo { x: 130 y : 40 controlX: 60 controlY: 10  }
            LineTo { x : 370, y : 40 }
            QuadCurveTo { x : 440 y : 70 controlX: 440 controlY: 10}
            LineTo { x : 500 y: 70 }
        ]
    }

    var animationPaths : Path[] = [ animationPathUpper, animationPathLower];
    


    def circleAnimations : PathTransition[] = for (n in [0..1]) {
        PathTransition {
            node: circles[n];
            path: AnimationPath.createFromPath(animationPaths[n]);
            interpolator: Interpolator.LINEAR
            autoReverse: false;
            repeatCount: Timeline.INDEFINITE;
            duration:  3s;
        }
    };



    def ResistorPath : Path = Path
    {
        translateX: 0 translateY : 0
        effect: DropShadow { offsetX : 3 offsetY : 3};
        fill : LinearGradient {
            proportional: true,
            startX: 0.0, startY: 0.0 endX: 0.0, endY: 1.0,
            stops: [
              Stop { offset: 0.0 color: Color.BEIGE },
              Stop { offset: 1.0 color: Color.ROSYBROWN }
              ]
        }

        elements: [
            MoveTo { x : 60 y: 70  }
            QuadCurveTo { x: 130 y : 100 controlX: 60 controlY: 130  }
            LineTo { x : 370, y : 100 }
            QuadCurveTo { x : 440 y : 70 controlX: 440 controlY: 130}
            QuadCurveTo { x : 370 y : 40 controlX: 440 controlY: 10}
            LineTo { x : 130, y : 40 }
            QuadCurveTo { x: 60 y : 70  controlX : 60 controlY : 10 }
            ClosePath { }
        ]
    };

    def animation: PathTransition = PathTransition {
        path : AnimationPath.createFromPath(ResistorPath),
    }

    public var Bands : ResistorBandShape[] =
    for (i in [0..<6])
        ResistorBandShape {
            xOffset : -10 + i * 35
    }
    
    protected override function create() : Node {
         circles[0].toFront();
         circles[1].toFront();
         circleAnimations[0].play();
         circleAnimations[1].play();
         
         return Group {
                 content: [
                    ResistorPath,

                    Line {
                        startX : 0 startY : 70 endX : 60 endY : 70
                        stroke : Color.BLACK strokeWidth : 2
                        effect: DropShadow { offsetX : 3 offsetY : 3};
                    },

                    Line {
                        startX : 440 startY : 70 endX : 500 endY : 70
                        stroke : Color.BLACK strokeWidth : 2
                        effect: DropShadow { offsetX : 3 offsetY : 3};
                    },

                    //Bands
                    Bands,

                    circles[0],
                    circles[1],

              ]
         }  // end Group
    } // end create()

}
